void checkSwitches()
{
  static byte previousstate[NUMBUTTONS];
  static byte currentstate[NUMBUTTONS];
  static long lasttime;
  byte index;

  if (millis() < lasttime)
  {
    // we wrapped around, lets just try again
    lasttime = millis();
  }

  if ((lasttime + DEBOUNCE) > millis()) {
    // not enough time has passed to debounce
    return;
  }
  // ok we have waited DEBOUNCE milliseconds, lets reset the timer
  lasttime = millis();

  for (index = 0; index < NUMBUTTONS; index++) // when we start, we clear out the "just" indicators
  {
    justreleased[index] = 0;
    justpressed[index] = 0;


    currentstate[index] = digitalRead(buttons[index]);   // read the button

    /*
    Serial.print(index, DEC);
    Serial.print(": cstate=");
    Serial.print(currentstate[index], DEC);
    Serial.print(", pstate=");
    Serial.print(previousstate[index], DEC);
    Serial.print(", press=");
    */

    if (currentstate[index] == previousstate[index]) {
      if ((pressed[index] == LOW) && (currentstate[index] == LOW)) {
        // just pressed
        justpressed[index] = 1;
      }
      else if ((pressed[index] == HIGH) && (currentstate[index] == HIGH)) {
        // just released
        justreleased[index] = 1;
      }
      pressed[index] = !currentstate[index];  // remember, digital HIGH means NOT pressed
    }
    //Serial.println(pressed[index], DEC);
    previousstate[index] = currentstate[index];   // keep a running tally of the buttons
  }
}

void clearJust()
{
  for (byte index = 0; index < NUMBUTTONS; index++) // when we start, we clear out the "just" indicators
  {
    justreleased[index] = 0;
    justpressed[index] = 0;
  }
}

void handlePresses()
{
  switch (menu)
  {
    case 0: // SPLASH/MAIN
      if (mainMenu == 0) // SYNTH
      {
        if (justpressed[13])
        {
          if (synPatchLoadSave == 0) // LOAD
            menu = 70;
          else // SAVE
            menu = 80;
          clearJust();
          valueChange = true;
          clearJust();
        }
      }
      checkKeyboard();
      break;
    case 10: // OSC1
      if (justpressed[13])
      {
        if (osc1WaveType == 4)
        {
          menu = 11;
          clearJust();
          valueChange = true;
        }
        else if (osc1WaveType == 3)
        {
          menu = 12;
          clearJust();
          valueChange = true;
          lcd.clear();
        }
      }
      else
        checkKeyboard();
      break;

    case 11: // OSC1 - choose user waveshape
      if (justpressed[12])
      {
        if (inFolder)
          setFolder();
        else
        {
          menu = 10;
          clearJust();
          valueChange = true;
          lcd.clear();
        }
      }
      else if (justpressed[13])
      {
        if (!inFolder)
          setFolder();
      }
      else
        checkKeyboard();
      break;

    case 12: // Squ Pulse Width
      if (justpressed[12])
      {
        menu = 10;
        clearJust();
        valueChange = true;
        lockPot(5);
        lcd.clear();
      }
      else
        checkKeyboard();
      break;

    case 20:
      if (justpressed[13])
      {
        if (osc2WaveType == 5)
        {
          menu = 21;
          clearJust();
          valueChange = true;
        }
        else if (osc2WaveType == 3)
        {
          menu = 22;
          clearJust();
          valueChange = true;
          lcd.clear();
        }
      }
      else
        checkKeyboard();
      break;

    case 21: // OSC2 - choose user waveshape
      if (justpressed[12])
      {
        if (inFolder)
          setFolder();
        else
        {
          menu = 20;
          clearJust();
          valueChange = true;
          lcd.clear();
        }
      }
      else if (justpressed[13])
      {
        if (!inFolder)
          setFolder();
      }
      else
        checkKeyboard();
      break;

    case 22: // Squ Pulse Width
      if (justpressed[12])
      {
        menu = 20;
        clearJust();
        valueChange = true;
        lockPot(5);
        lcd.clear();
      }
      else
        checkKeyboard();
      break;

    case 30:
      checkKeyboard();
      break;

    case 40:
      checkKeyboard();
      break;

    case 50:
      if (justpressed[13])
      {
        if (lfoShapePointer == &nUserTable3[0])
        {
          menu = 51;
          clearJust();
          valueChange = true;
        }
      }
      else
        checkKeyboard();
      break;

    case 51: // LFO - choose user waveshape
      if (justpressed[12])
      {
        if (inFolder)
          setFolder();
        else
        {
          menu = 50;
          clearJust();
          valueChange = true;
          lcd.clear();
        }
      }
      else if (justpressed[13])
      {
        if (!inFolder)
          setFolder();
      }
      else
        checkKeyboard();
      break;

    case 60:
      checkKeyboard();
      break;

    case 70: // LOAD
      if (justpressed[12])
      {
        if (inFolder)
          setFolder();
        else
        {
          menu = 0;
          valueChange = true;
          lcd.clear();
          updateMenu();
        }
        lockPot(5);
        clearJust();
      }
      else if (justpressed[13])
      {
        if (!inFolder)
          setFolder();
        lockPot(5);
        clearJust();
      }
      else
        checkKeyboard();
      break;

    case 80: // SAVE
      if (justpressed[12])
      {
        if (inFolder)
          setFolder();
        else
        {
          menu = 0;
          valueChange = true;
          lcd.clear();
          updateMenu();
        }
        lockPot(5);
        clearJust();
      }
      else if (justpressed[13])
      {
        if (!inFolder)
          setFolder();
        else if (saveConfirm)
        {
          saveConfirm = false;
          savePatch();
        }
        else
        {
          if (strcmp(fileName, saveName) == 0)
            savePatch();
          else
          {
            saveConfirm = true;
            lcd.setCursor(4, 1);
            lcd.print("Overwrite?  ");
          }
        }
        lockPot(5);
        clearJust();
      }
      else
        checkKeyboard();
      //savePatch();
      //menu = 0;
      break;
  }
}

void checkKeyboard()
{
  // debug
  for (byte i = 0; i < 13; i++)
  {
    if (justpressed[i])
    {
      if (midiMode) // switch to listening to notes from front-panel keyboard
      {
        midiMode = false;
        monoMode = false;
        filterCutoff = lastFilterCutoff;
        envelopeProgress = 255;
        for (byte j = 0; j < 4; j++)
          voice[j] = 255;
        for (byte k = 0; k < 13; k++)
          keyAssigned[k] = false;
      }
      LEDon = true;
      clearJust();
    }
  }
  if (!midiMode)
  {
    voiceCounter = 4;
    for (byte i = 0; i < 4; i++) // check if a previously held note has been released
    {
      if (voice[i] != 255)
      {
        if (!pressed[voice[i]])
        {
          keyAssigned[voice[i]] = false;
          voice[i] = 255;
          voiceCounter--;
        }
      }
      else //
        voiceCounter--;
    }

    if (voiceCounter == 0 && lastVoiceCount != 0)
    {
      envelopeProgress = 3; // we're in release phase
      releaseStartTime = millis();
      LEDon = true;
    }

    for (byte i = 0; i < 13; i++) // assign note to empty voice
    {
      if (!keyAssigned[i]) // if this key isn't already assigned, look for an empty voice for it
      {
        if (pressed[i])
        {
          byte j = 0;
          while (voice[j] != 255 && j < 4) // go to the next voice if this voice isn't free
            j++;
          if (j < 4)
          {
            voice[j] = i;
            keyAssigned[i] = true;
            envelopeTrigger = true;
            if (retrigger) // should we retrigger the LFO?
              lfoIndex = 0;
            if (voiceCounter == 0)
            {
              voiceCounter++;
              for (byte k = 0; k < 4; k++)
              {
                if (k != j)
                  muteVoice[k] = true;
                //voiceSounding[k] = false;
              }
            }
          }
        }
      }
    }

    lastVoiceCount = voiceCounter;
    /*
      doOscSpread = true;
      if (noteCounter == 0) // no notes are held
      {
        envelopeTrigger == false;
        LEDon = false;
        //envelopeVolume = 0;
        doOscSpread = false;
      }
    */
  }
}
