void getPots() // read the current pot values
{
  for (byte i = 0; i < 5; i++)
  {
    pot[i] = analogRead(i);
  }

  // see if it's time to lock the pots
  if (lockTimer != 0 && (millis() - lockTimer) > 500) // the pots lock after 500ms
  {
    lockPot(5);
    valueChange = true;
  }
}

void lockPot(byte Pot) // values of 0 - 4 for each pot and 5 for all
{
  if (Pot == 5)
  {
    for (byte i = 0; i < 5; i++)
      potLock[i] = pot[i];
    lockTimer = 0;
  }
  else
    potLock[Pot] = pot[Pot];
}

boolean unlockedPot(byte Pot) // check if a pot is locked or not
{
  if (potLock[Pot] == 9999)
  {
    if (abs(pot[Pot] - lastPotValue[Pot]) > 10) // the threshold value is 10
    {
      lastPotValue[Pot] = pot[Pot];
      lockTimer = millis();
      if (Pot < 4)
        valueChange = true;
    }
    return true;
  }
  else if (abs(potLock[Pot] - pot[Pot]) > 10) // the threshold value is 10
  {
    potLock[Pot] = 9999;
    lockTimer = millis();
    lastPotValue[Pot] = pot[Pot];
    return true;
  }
  else
    return false;
}

void adjustValues()
{
  switch (menu)
  {
    case 0: // main
      if (unlockedPot(0))
      {
        if (splash)
          {
            customCharacters();
            splash = false;
          }
        if (pot[0] < 1023 / 4) // there are 4 mainMenu items
        {
          if (mainMenu != 0)
          {
            mainMenu = 0;
            updateMenu();
          }
        }
        else if (pot[0] < (1023 / 4) * 2)
        {
          if (mainMenu != 1)
          {
            mainMenu = 1;
            updateMenu();
          }
        }
        else if (pot[0] < (1023 / 4) * 3)
        {
          if (mainMenu != 2)
          {
            mainMenu = 2;
            updateMenu();
          }
        }
        else
        {
          if (mainMenu != 3)
          {
            mainMenu = 3;
            updateMenu();
          }
        }
      }

      if (unlockedPot(1))
      {
        if (splash)
          {
            customCharacters();
            splash = false;
          }
        if (pot[1] < 512)
        {
          if (synPatchLoadSave != 0)
          {
            synPatchLoadSave = 0;
            updateMenu();
          }
        }
        else
        {
          if (synPatchLoadSave != 1)
          {
            synPatchLoadSave = 1;
            updateMenu();
          }
        }
      }
      break;
    case 10: // OSC1
      if (unlockedPot(0))
      {
        if (pot[0] < (1023 / waveshapes))
          setOsc1WaveType(0); // sine
        else if (pot[0] < (1023 / waveshapes) * 2)
          setOsc1WaveType(1); // triangle
        else if (pot[0] < (1023 / waveshapes) * 3)
          setOsc1WaveType(2); // saw
        else if (pot[0] < (1023 / waveshapes) * 4)
          setOsc1WaveType(3); // square
        else if (pot[0] < (1023 / waveshapes) * 5)
          setOsc1WaveType(4); // user1
        else if (pot[0] < (1023 / waveshapes) * 6)
          setOsc1WaveType(5); // user2
        else if (pot[0] < (1023 / waveshapes) * 7)
          setOsc1WaveType(6); // user3
        else
          setOsc1WaveType(7); // noise
      }

      if (unlockedPot(1))
      {
        osc1Octave = map(pot[1], 0, 1023, 1, 9);
        if (osc1Octave != lastOsc1Octave)
        {
          lastOsc1Octave = osc1Octave;
          assignVoices();
        }
      }

      if (unlockedPot(2))
      {
        osc1Volume = pot[2];
      }

      if (unlockedPot(3))
      {
        if (pot[3] > 502 && pot[3] < 522)
          osc1Detune = 0;
        else
          osc1Detune = int(pot[3] - 512) << 11;
        if (osc1Detune != lastOsc1Detune)
        {
          lastOsc1Detune = osc1Detune;
          assignVoices();
        }
      }
      break;

    case 11: // OSCILLATORS - set user wave shapes
      if (unlockedPot(3) && dirCount != 0)
      {
        dirChoice = map(pot[3], 0, 1023, 1, dirCount);
        if (dirChoice != lastDirChoice)
        {
          lastDirChoice = dirChoice;
          sd.vwd()->rewind();
          tempCount = 0;
          waveshapeLoaded = false;
        }
      }
      break;

    case 12: // Squ: Pulse Width
      if (unlockedPot(3))
      {
        //valueChange = true;
        uiPulseWidth = map(pot[3], 0, 1023, ((WAVE_SAMPLES / 2) - 15) * -1, ((WAVE_SAMPLES / 2) - 15));
      }
      break;

    case 20: // OSC2
      {
        if (unlockedPot(0))
        {
          if (pot[0] < (1023 / waveshapes))
            setOsc2WaveType(0); // sine
          else if (pot[0] < (1023 / waveshapes) * 2)
            setOsc2WaveType(1); // triangle
          else if (pot[0] < (1023 / waveshapes) * 3)
            setOsc2WaveType(2); // saw
          else if (pot[0] < (1023 / waveshapes) * 4)
            setOsc2WaveType(3); // square
          else if (pot[0] < (1023 / waveshapes) * 5)
            setOsc2WaveType(4); // user1
          else if (pot[0] < (1023 / waveshapes) * 6)
            setOsc2WaveType(5); // user2
          else if (pot[0] < (1023 / waveshapes) * 7)
            setOsc2WaveType(6); // user3
          else
            setOsc2WaveType(7); // noise
        }

        if (unlockedPot(1))
        {
          osc2Octave = map(pot[1], 0, 1023, 1, 9);
          if (osc2Octave != lastOsc2Octave)
          {
            lastOsc2Octave = osc2Octave;
            assignVoices();
          }
        }

        if (unlockedPot(2))
        {
          osc2Volume = pot[2];
        }

        if (unlockedPot(3))
        {
          if (pot[3] > 502 && pot[3] < 522)
            osc2Detune = 0;
          else
            osc2Detune = int(pot[3] - 512) << 11;
          if (osc2Detune != lastOsc2Detune)
          {
            lastOsc2Detune = osc2Detune;
            assignVoices();
          }
        }
      }
      break;

    case 21: // OSCILLATORS - set user wave shapes
      if (unlockedPot(3) && dirCount != 0)
      {
        dirChoice = map(pot[3], 0, 1023, 1, dirCount);
        if (dirChoice != lastDirChoice)
        {
          lastDirChoice = dirChoice;
          sd.vwd()->rewind();
          tempCount = 0;
          waveshapeLoaded = false;
        }
      }
      break;

    case 22: // Squ: Pulse Width
      if (unlockedPot(3))
      {
        //valueChange = true;
        uiPulseWidth = map(pot[3], 0, 1023, ((WAVE_SAMPLES / 2) - 15) * -1, ((WAVE_SAMPLES / 2) - 15));
      }
      break;

    case 30: // FILTER
      if (unlockedPot(0))
      {
        //valueChange = true;
        float sensVar = (pot[0] >> 2) * 1.0;
        filterCutoff = pow(sensVar / 10, 1.71); //(base, exponent)
      }
      if (unlockedPot(1))
      {
        filterResonance = pot[1] >> 2;
        setFilterResonance(filterResonance);
      }

      if (unlockedPot(2))
      {
        if (pot[2] < 341)
          filterType(0);
        else if (pot[2] < 682)
          filterType(1);
        else
          filterType(2);
      }

      if (unlockedPot(3))
      {
        if (pot[3] < 512)
          filterBypass = true;
        else
          filterBypass = false;
      }

      break;

    case 40: // ENVELOPE
      if (unlockedPot(0))
      {
        attackTime = constrain(pot[0], 1, 1023); // we don't want to be able to set 0
      }
      if (unlockedPot(1))
      {
        decayTime = constrain(pot[1], 1, 1023); // we don't want to be able to set 0
      }
      if (unlockedPot(2))
      {
        sustainLevel = pot[2];
      }
      if (unlockedPot(3))
      {
        releaseTime = constrain(pot[3], 1, 1023); // we don't want to be able to set 0
      }
      break;

    case 50: // LFO
      if (unlockedPot(0))
      {
        if (pot[0] < (1023 / 7))
        {
          if (lfoShape != 0)
          {
            lfoShape = 0; // sine
            setLfoShape(lfoShape);
          }
        }
        else if (pot[0] < (1023 / 7) * 2)
        {
          if (lfoShape != 1)
          {
            lfoShape = 1; // triangle
            setLfoShape(lfoShape);
          }
        }
        else if (pot[0] < (1023 / 7) * 3)
        {
          if (lfoShape != 2)
          {
            lfoShape = 2; // saw
            setLfoShape(lfoShape);
          }
        }
        else if (pot[0] < (1023 / 7) * 4)
        {
          if (lfoShape != 3)
          {
            lfoShape = 3; // square
            setLfoShape(lfoShape);
          }
        }
        else if (pot[0] < (1023 / 7) * 5)
        {
          if (lfoShape != 4)
          {
            lfoShape = 4; // user1
            setLfoShape(lfoShape);
          }
        }
        else if (pot[0] < (1023 / 7) * 6)
        {
          if (lfoShape != 5)
          {
            lfoShape = 5; // user2
            setLfoShape(lfoShape);
          }
        }
        else
        {
          if (lfoShape != 0)
          {
            {
              lfoShape = 6; // user3
              setLfoShape(lfoShape);
            }
          }
        }
      }
      if (unlockedPot(1))
      {
        if (lfoLowRange)
          userLfoRate = 63 - (pot[1] >> 4);
        else // high range
          userLfoRate = 1024 - (pot[1]); // 1024 here because we don't want the rate to ever be 0;
        lfoRate = userLfoRate;
      }
      if (unlockedPot(2))
      {
        if (pot[2] < 512)
        {
          if (lfoRate > 31)
            lfoRate = 31;
          lfoLowRange = true;
        }
        else
          lfoLowRange = false;
      }
      if (unlockedPot(3))
      {
        if (pot[3] < 512)
          retrigger = true;
        else
          retrigger = false;
      }
      break;

    case 51: // LFO - set user wave shapes
      if (unlockedPot(3) && dirCount != 0)
      {
        dirChoice = map(pot[3], 0, 1023, 1, dirCount);
        if (dirChoice != lastDirChoice)
        {
          lastDirChoice = dirChoice;
          sd.vwd()->rewind();
          tempCount = 0;
          waveshapeLoaded = false;
        }
      }
      break;

    case 60: // MODULATION
      if (unlockedPot(0))
      {
        if (pot[0] < 512)
          source = 0;
        else
          source = 1;
      }
      if (unlockedPot(1))
      {
        switch (source)
        {
          case 0: // SOURCE LFO
            dest = 1023 / 7; // how many destinations are there
            if (pot[1] < dest)
              destination = 0;
            else if (pot[1] < dest * 2)
              destination = 1;
            else if (pot[1] < dest * 3)
              destination = 2;
            else if (pot[1] < dest * 4)
              destination = 3;
            else if (pot[1] < dest * 5)
              destination = 4;
            else if (pot[1] < dest * 6)
              destination = 5;
            else
              destination = 6;
            break;

          case 1: // SOURCE ENVELOPE
            dest = 1023 / 4; // how many destinations are there
            if (pot[1] < dest)
              destination = 7;
            else if (pot[1] < dest * 2)
              destination = 8;
            else if (pot[1] < dest * 3)
              destination = 9;
            else
              destination = 10;
            break;
        }
      }
      if (unlockedPot(2))
      {
        switch (source)
        {
          case 0: // SOURCE LFO
            switch (destination)
            {
              case 0:
                lfoOsc1DetuneFactor = pot[2];
                break;
              case 1:
                lfoOsc2DetuneFactor = pot[2];
                break;
              case 2:
                osc1OctaveMod = pot[2] >> 8;
                break;
              case 3:
                osc2OctaveMod = pot[2] >> 8;
                break;
              case 4:
                lfoAmount = pot[2];
                break;
              case 5:
                lfoAmpFactor = pot[2];
                break;
              case 6:
                lfoPwFactor = pot[2];
                break;
            }
            break;

          case 1: // SOURCE ENVELOPE
            switch (destination)
            {
              case 7:
                if (pot[2] > 502 && pot[2] < 522) // snap to center / 0
                  envOsc1PitchFactor = 0;
                else
                  envOsc1PitchFactor = (pot[2] - 512) << 1; // - 1023 to 1023
                break;
              case 8:
                if (pot[2] > 502 && pot[2] < 522) // snap to center / 0
                  envOsc2PitchFactor = 0;
                else
                  envOsc2PitchFactor = (pot[2] - 512) << 1; // - 1023 to 1023
              case 9: // FILTER CUTOFF
                if (pot[2] > 502 && pot[2] < 522) // snap to center / 0
                  envFilterCutoffFactor = 0;
                else
                  envFilterCutoffFactor = (pot[2] - 512) << 1; // - 1023 to 1023
                break;
              case 10: // LFO RATE
                envLfoRate = pot[2];
                break;
            }
            break;
        }
      }
      break;
    case 70: // LOAD SYNTH PATCH
      if (unlockedPot(3) && dirCount != 0)
      {
        dirChoice = map(pot[3], 0, 1023, 1, dirCount);
        if (dirChoice != lastDirChoice)
        {
          lastDirChoice = dirChoice;
          sd.vwd()->rewind();
          tempCount = 0;
        }
      }
      break;

    case 80: // SAVE SYNTH PATCH
      if (unlockedPot(3) && dirCount != 0)
      {
        dirChoice = map(pot[3], 0, 1023, 1, dirCount);
        if (dirChoice != lastDirChoice)
        {
          lastDirChoice = dirChoice;
          sd.vwd()->rewind();
          tempCount = 0;
        }
      }    
      break;
  }
}

void getMenu()
{
  switch (mainMenu)
  {
    case 0: // SYNTH
      byte menuPages = 7;
      static byte lastMenu = 0;
      if (unlockedPot(4)) // select the menu page
      {
        if (splash)
          {
            customCharacters();
            splash = false;
          }
        int tmp = 1023 / menuPages;
        if (pot[4] < 10)
        {
          menu = 0;
        }
        else if (pot[4] < tmp)
          menu = 10; // OSC1
        else if (pot[4] < (2 * tmp))
          menu = 20; // OSC2
        else if (pot[4] < (3 * tmp))
          menu = 30; // FILTER
        else if (pot[4] < (4 * tmp))
          menu = 40; // ENVELOPE
        else if (pot[4] < (5 * tmp))
          menu = 50; // LFO
        else
          menu = 60; // MODULATION
        if (menu != lastMenu)
        {
          lastMenu = menu;
          for (byte i = 0; i < 4; i++)
            lockPot(i);
          updateMenu();
        }
      }
      if (menu == 50) // lfo page
        lfoLED = true;
      else
        lfoLED = false;
      break;
  }
}

