// *** WAVE FILE HEADER ***
typedef struct {
  char RIFF[4];
  int32_t chunk_size;
  char WAVE[4];
  char fmt_[4];
  int32_t subchunk1_size;
  int16_t audio_format;
  int16_t num_channels;
  int32_t sample_rate;
  int32_t byte_rate;
  int16_t block_align;
  int16_t bits_per_sample;
  char DATA[4];
  int32_t subchunk2_size;
} wave_header;

wave_header header;

void getWaveform()
{
  if (dirCount == 0 && dirChecked == false)
  {
    static uint16_t tempCount = 0;
    while (file.openNext(sd.vwd(), O_READ))
    {
      file.close();
      dirCount++;
    }
    dirChoice = 0;
    sd.vwd()->rewind();
    dirChecked = true;
    valueChange = true;
  }

  else
  {
    while (tempCount < dirChoice)
    {
      file.openNext(sd.vwd(), O_READ);
      tempCount++;
      file.getFilename(fileName);
      if (file.isDir())
      {
        folder = true;
        file.getFilename(folderName);
      }
      else
        folder = false;
      file.close();
    }
    if (tempCount == dirChoice)
    {
      lcd.setCursor(0, 1);
      lcd.print("    ");
      lcd.print(fileName);
      if (folder)
        arrow(strlen(fileName) + 4, 1);
      //lcd.print(char(127));
      lcd.print("            "); // sloppy way to clear excess characters
    }
    if (!waveshapeLoaded)
    {
      // if (
      if (checkExtension(".WAV"))
        //);
        loadWaveshape();
    }
  }
}

void loadWaveshape()
{
  if (!folder)
  {
    file.open(fileName);
    file.seekSet(44); // just after the header data
    file.read(waveShapeBuffer, 1200); // we know our waveeshape files are all 1200 bytes long (ie. 600 * 16 bit ints)
    file.close();
    switch (menu)
    {
      case 11:
        for (int i = 0; i < 600; i ++)
          nUserTable1[i] = (waveShapeBuffer[i] >> 4) + 2048;
        break;
      case 21:
        for (int i = 0; i < 600; i ++)
          nUserTable2[i] = (waveShapeBuffer[i] >> 4) + 2048;
        break;
      case 51:
        for (int i = 0; i < 600; i ++)
          nUserTable3[i] = (waveShapeBuffer[i] >> 4) + 2048;
        break;
    }
    file.close();
    waveshapeLoaded = true;
  }
}

boolean checkExtension(char ext[]) // checks the extension of the file, use argument like ".WAV", ".TB2"
{
  byte nameLength = strlen(fileName);
  if (nameLength < 6)
    return false;
  else
  {
    char lastFour[5];
    for (byte i = 0; i < 5; i++)
      lastFour[i] = fileName[nameLength - 4 + i]; // create a string with only the last four chars
    if (strcmp(ext, lastFour) == 0)
      return true;
    else
      return false;
  }
}

void savePatch()
{
  //sd.chdir();
  file.open(fileName, O_RDWR | O_CREAT); // create file if it doesn't exist and open the file for write
  // OSC1
  patchBuffer[0] = osc1WaveType;
  patchBuffer[1] = osc1Octave;
  patchBuffer[2] = osc1Volume;
  patchBuffer[3] = osc1Detune;
  patchBuffer[4] = pulseWidth;
  // OSC2
  patchBuffer[5] = osc2WaveType;
  patchBuffer[6] = osc2Octave;
  patchBuffer[7] = osc2Volume;
  patchBuffer[8] = osc2Detune;
  // FILTER
  patchBuffer[9] = filterCutoff;
  patchBuffer[10] = filterResonance;
  patchBuffer[11] = fType;
  patchBuffer[12] = filterBypass;
  // ENVELOPE
  patchBuffer[13] = attackTime;
  patchBuffer[14] = decayTime;
  patchBuffer[15] = sustainLevel;
  patchBuffer[16] = releaseTime;
  // LFO
  patchBuffer[17] = lfoShape;
  patchBuffer[18] = lfoRate;
  patchBuffer[19] = lfoLowRange;
  patchBuffer[20] = retrigger;
  // MODULATION
  patchBuffer[21] = lfoOsc1DetuneFactor;
  patchBuffer[22] = lfoOsc2DetuneFactor;
  patchBuffer[23] = osc1OctaveMod;
  patchBuffer[24] = osc2OctaveMod;
  patchBuffer[25] = lfoAmount;
  patchBuffer[26] = lfoAmpFactor;
  patchBuffer[27] = lfoPwFactor;
  patchBuffer[28] = envOsc1PitchFactor;
  patchBuffer[29] = envOsc2PitchFactor;
  patchBuffer[30] = envFilterCutoffFactor;
  patchBuffer[31] = envLfoRate;
  // write zeros to 100 for future use
  for (byte i = 32; i < 100; i++)
    patchBuffer[i] = 0;
  // USER 1 WAVESHAPE
  for (uint16_t i = 0; i < 600; i++)
    patchBuffer[i + 100] = nUserTable1[i];
  // USER 2 WAVESHAPE
  for (uint16_t i = 0; i < 600; i++)
    patchBuffer[i + 700] = nUserTable2[i];
  // USER 3 WAVESHAPE
  for (uint16_t i = 0; i < 600; i++)
    patchBuffer[i + 1300] = nUserTable3[i];
  // WRITE TO SD
  if (file.write(patchBuffer, 7600) != -1) // note - we are writing 1900 4 byte ints from the patch buffer to 7600 bytes on the SD
  {
   // lcd.setCursor(0, 1);
   // lcd.print("WRITTEN!");
    if (file.sync())
    {
      lcd.setCursor(4, 1);
      lcd.print("SAVED!      ");
    }
  }
  file.close();
}

void loadPatch()
{
  //sd.chdir();
  //file.open("PATCHES1/01.TB2");
  file.open(fileName);
  if (file.read(patchBuffer, 7600) == 7600) // note - we are reading 7600 bytes to a buffer of 1900 4 byte ints
  {
    // OSC1
    osc1WaveType = patchBuffer[0];
    setOsc1WaveType(osc1WaveType);
    osc1Octave = patchBuffer[1];
    osc1Volume = patchBuffer[2];
    osc1Detune = patchBuffer[3];
    pulseWidth = patchBuffer[4];
    uiPulseWidth = pulseWidth;
    // OSC2
    osc2WaveType = patchBuffer[5];
    setOsc2WaveType(osc2WaveType);
    osc2Octave = patchBuffer[6];
    osc2Volume = patchBuffer[7];
    osc2Detune = patchBuffer[8];
    // FILTER
    filterCutoff = patchBuffer[9];
    filterResonance = patchBuffer[10];
    setFilterResonance(filterResonance);
    fType = patchBuffer[11];
    filterBypass = patchBuffer[12];
    // ENVELOPE
    attackTime = patchBuffer[13];
    decayTime = patchBuffer[14];
    sustainLevel = patchBuffer[15];
    releaseTime = patchBuffer[16];
    // LFO
    lfoShape  = patchBuffer[17];
    setLfoShape(lfoShape);
    lfoRate = patchBuffer[18];
    userLfoRate = lfoRate;
    lfoLowRange = patchBuffer[19];
    retrigger = patchBuffer[20];
    // MODULATION
    lfoOsc1DetuneFactor = patchBuffer[21];
    lfoOsc2DetuneFactor = patchBuffer[22];
    osc1OctaveMod = patchBuffer[23];
    osc2OctaveMod = patchBuffer[24];
    lfoAmount = patchBuffer[25];
    lfoAmpFactor = patchBuffer[26];
    lfoPwFactor = patchBuffer[27];
    envOsc1PitchFactor = patchBuffer[28];
    envOsc2PitchFactor = patchBuffer[29];
    envFilterCutoffFactor = patchBuffer[30];
    envLfoRate = patchBuffer[31];
    // USER 1 WAVESHAPE
    for (uint16_t i = 0; i < 600; i++)
      nUserTable1[i] = patchBuffer[i + 100];
    // USER 2 WAVESHAPE
    for (uint16_t i = 0; i < 600; i++)
      nUserTable2[i] = patchBuffer[i + 700];
    // USER 3 WAVESHAPE
    for (uint16_t i = 0; i < 600; i++)
      nUserTable3[i] = patchBuffer[i + 1300];
  }
  file.close();
}

void setFolder()
{
  dirCount = 0;
  dirChecked = false;
  switch (inFolder)
  {
    case true:
      inFolder = false;
      sd.chdir(); // change current directory to root
      sd.vwd()->rewind();
      clearJust();
      valueChange = true;
      break;

    case false:
      inFolder = true;
      sd.chdir(folderName); // change current directory to folderName
      sd.vwd()->rewind();
      clearJust();
      valueChange = true;
      break;
  }
}

void getSynthPatch()
{
  if (dirCount == 0 && dirChecked == false)
  {
    static uint16_t tempCount = 0;
    while (file.openNext(sd.vwd(), O_READ))
    {
      file.close();
      dirCount++;
    }
    dirChoice = 0;
    sd.vwd()->rewind();
    dirChecked = true;
    valueChange = true;
  }

  else
  {
    while (tempCount < dirChoice)
    {
      file.openNext(sd.vwd(), O_READ);
      tempCount++;
      file.getFilename(fileName);
      if (file.isDir())
      {
        folder = true;
        file.getFilename(folderName);
      }
      else
        folder = false;
      file.close();
    }
    if (tempCount == dirChoice)
    {
      lcd.setCursor(0, 1);
      lcd.print("    ");
      lcd.print(fileName);
      if (folder)
        arrow(strlen(fileName) + 4, 1);
      //lcd.print(char(127));
      lcd.print("            "); // sloppy way to clear excess characters
    }
    if (!synthPatchLoaded)
    {
      if (checkExtension(".TB2"))
        loadPatch();
    }
  }
}

void saveSynthPatch()
{
  if (dirCount == 0 && dirChecked == false)
  {
    static uint16_t tempCount = 0;
    while (file.openNext(sd.vwd(), O_READ))
    {
      file.close();
      dirCount++;
    }
    dirChoice = 0;
    sd.vwd()->rewind();
    dirChecked = true;
    valueChange = true;
    if (dirCount < 98)
    {
      numberName = dirCount + 1;
      if (numberName < 10)
      {
        //saveName[0] = 0x30; // "0" in hex
        sprintf(saveName, "0%d.TB2", numberName);
      }
      else
      {
        sprintf(saveName, "%d.TB2", numberName);
      }
      lcd.setCursor(0, 1);
      lcd.print("    ");
      lcd.print(saveName);
      lcd.print("      ");
      strcpy(fileName, saveName);
    }
    else
    {
      lcd.setCursor(0, 1);
      lcd.print("    Folder Full!");
    }
  }

  else
  {
    while (tempCount < dirChoice)
    {
      file.openNext(sd.vwd(), O_READ);
      tempCount++;
      file.getFilename(fileName);
      if (file.isDir())
      {
        folder = true;
        file.getFilename(folderName);
      }
      else
        folder = false;
      file.close();
    }
    if (tempCount == dirChoice)
    {
      lcd.setCursor(0, 1);
      lcd.print("    ");
      lcd.print(fileName);
      if (folder)
        arrow(strlen(fileName) + 4, 1);
      //lcd.print(char(127));
      lcd.print("            "); // sloppy way to clear excess characters
    }
    //    if (!synthPatchLoaded)
    //    {
    //      if (checkExtension(".TB2"))
    //        loadPatch();
    //    }
  }
}


void getFirstFile()
{
  sd.vwd()->rewind();
  file.openNext(sd.vwd(), O_READ);
  file.getFilename(fileName);
  if (file.isDir())
  {
    folder = true;
    file.getFilename(folderName);
  }
  file.close();
}
