// *** LED ***
void updateLED()
{
  if (lfoLED)
    analogWrite(LED, tmpLFO >> 4);
  else if (LEDon)
    analogWrite(LED, 255);
  else
    analogWrite(LED, 0);
}

// *** LCD ***
void showValue(byte h, byte v, int number) // a function to help us deal with displaying numbers of varying lengths
{
  if (number < 10)
  {
    lcd.setCursor(h, v);
    lcd.print(number);
    lcd.print("  ");
  }
  else if (number < 100)
  {
    lcd.setCursor(h, v);
    lcd.print(number);
    lcd.print(" ");
  }
  else
  {
    lcd.setCursor(h, v);
    lcd.print(number);
  }
}

// *** UI ***
void updateMenu()
{
  arrowAnimation = false;
  switch (menu)
  {
    case 0: // SPLASH/MAIN
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("MAIN            ");
      switch (mainMenu)
      {
        case 0:
          lcd.setCursor(0, 1);
          lcd.print("Synth           ");
          lcd.setCursor(6, 0);
          lcd.print("PATCH     ");
          if (synPatchLoadSave == 0) // load
          {
            lcd.setCursor(6, 1);
            lcd.print("Load      ");
          }
          else // save
          {
            lcd.setCursor(6, 1);
            lcd.print("Save      ");
          }
          //lcd.setCursor(10, 1);
          //lcd.write(byte(0));
          arrow(10, 1);
          break;
        case 1:
          lcd.setCursor(0, 1);
          lcd.print("Arp             ");
          break;
        case 2:
          lcd.setCursor(0, 1);
          lcd.print("Seq             ");
          break;
        case 3:
          lcd.setCursor(0, 1);
          lcd.print("Preferences     ");
          break;
      }
      break;
    case 10:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("OSCILLATORS     ");
      lcd.setCursor(0, 1);
      lcd.print("OSC1            ");
      break;
    case 20:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("OSCILLATORS     ");
      lcd.setCursor(0, 1);
      lcd.print("OSC2            ");
      break;
    case 30:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("FILTER          ");
      break;
    case 40:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("ENVELOPE (ADSR) ");
      break;
    case 50:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("LFO             ");
      break;
    case 60:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("MODULATION      ");
      break;
  }
}

void updateValues()
{
  if (uiRefresh && valueChange && menu != 0)
  {
    arrowAnimation = false;
    uiRefresh = false;
    valueChange = false;
    switch (menu)
    {
      case 0: // SPLASH/MAIN
        lcd.setCursor(0, 0);
        lcd.print("Main            ");
        break;

      case 10: // OSC1
        lcd.setCursor(0, 0);
        lcd.print("Os1 Oct Lvl Det ");

        switch (osc1WaveType)
        {
          case 0:
            lcd.setCursor(0, 1);
            lcd.print("Sin ");
            break;
          case 1:
            lcd.setCursor(0, 1);
            lcd.print("Tri ");
            break;
          case 2:
            lcd.setCursor(0, 1);
            lcd.print("Saw ");
            break;
          case 3:
            lcd.setCursor(0, 1);
            lcd.print("Squ ");
            arrow(3, 1);
            break;
          case 4:
            lcd.setCursor(0, 1);
            lcd.print("Us1 ");
            arrow(3, 1);
            break;
          case 5:
            lcd.setCursor(0, 1);
            lcd.print("Us2 ");
            break;
          case 6:
            lcd.setCursor(0, 1);
            lcd.print("Us3 ");
            break;
          case 7:
            lcd.setCursor(0, 1);
            lcd.print("Nse ");
            break;
        }

        showValue(4, 1, osc1Octave); // octave

        showValue(8, 1, osc1Volume >> 2); // level

        showValue(12, 1, osc1Detune >> 14); // detune
        break;

      case 11: // OSCILLATORS - change user waveshape
        lcd.setCursor(0, 0);
        lcd.print("Us1: ");

        if (inFolder)
        {
          lcd.setCursor(4, 0);
          lcd.print(folderName);
          lcd.print("        ");
        }
        else
        {
          lcd.setCursor(4, 0);
          lcd.print("Root");
          lcd.print("        ");
          lcd.setCursor(0, 1);
          lcd.print("    Choose [P4] ");
        }
        getWaveform();
        break;

      case 12: // SQUARE - adjust PWM
        lcd.setCursor(0, 0);
        lcd.print("Squ: Pulse Width");
        showValue(11, 1, (map(uiPulseWidth, ((WAVE_SAMPLES / 2) - 15) * -1, ((WAVE_SAMPLES / 2) - 15), 0, 255)));
        break;

      case 20: // OSC2
        lcd.setCursor(0, 0);
        lcd.print("Os2 Oct Lvl Det ");

        switch (osc2WaveType)
        {
          case 0:
            lcd.setCursor(0, 1);
            lcd.print("Sin ");
            break;
          case 1:
            lcd.setCursor(0, 1);
            lcd.print("Tri ");
            break;
          case 2:
            lcd.setCursor(0, 1);
            lcd.print("Saw ");
            break;
          case 3:
            lcd.setCursor(0, 1);
            lcd.print("Squ ");
            arrow(3, 1);
            break;
          case 4:
            lcd.setCursor(0, 1);
            lcd.print("Us1 ");
            break;
          case 5:
            lcd.setCursor(0, 1);
            lcd.print("Us2 ");
            arrow(3, 1);
            break;
          case 6:
            lcd.setCursor(0, 1);
            lcd.print("Us3 ");
            break;
          case 7:
            lcd.setCursor(0, 1);
            lcd.print("Nse ");
            break;
        }
        showValue(4, 1, osc2Octave); // octave

        showValue(8, 1, osc2Volume >> 2); // level

        showValue(12, 1, osc2Detune >> 14); // detune
        break;

      case 21: // OSC2 - change user waveshape
        lcd.setCursor(0, 0);
        lcd.print("Us2: ");

        if (inFolder)
        {
          lcd.setCursor(4, 0);
          lcd.print(folderName);
          lcd.print("        ");
        }
        else
        {
          lcd.setCursor(4, 0);
          lcd.print("Root");
          lcd.print("        ");
          lcd.setCursor(0, 1);
          lcd.print("    Choose [P4] ");
        }
        getWaveform();
        break;

      case 22: // SQUARE - adjust PWM
        lcd.setCursor(0, 0);
        lcd.print("Squ: Pulse Width");

        showValue(11, 1, (map(uiPulseWidth, ((WAVE_SAMPLES / 2) - 15) * -1, ((WAVE_SAMPLES / 2) - 15), 0, 255)));
        break;

      case 30: // FILTER
        lcd.setCursor(0, 0);
        lcd.print("Cut Res Typ Byp");
        showValue(0, 1, filterCutoff);
        showValue(4, 1, filterResonance);
        lcd.setCursor(12, 1);
        switch (fType)
        {
          case 0:
            lcd.setCursor(8, 1);
            lcd.print("LP  ");
            break;
          case 1:
            lcd.setCursor(8, 1);
            lcd.print("BP  ");
            break;
          case 2:
            lcd.setCursor(8, 1);
            lcd.print("HP  ");
            break;
        }
        if (filterBypass)
        {
          lcd.print("On ");
        }
        else
          lcd.print("Off");
        break;

      case 40: // ENVELOPE
        lcd.setCursor(0, 0);
        lcd.print("Att Dec Sus Rel ");
        showValue(0, 1, attackTime >> 2);
        showValue(4, 1, decayTime >> 2);
        showValue(8, 1, sustainLevel >> 2);
        showValue(12, 1, releaseTime >> 2);
        break;

      case 50: // LFO
        lcd.setCursor(0, 0);
        lcd.print("Shp Rt  Rng Trg");
        switch (lfoShape)
        {
          case 0:
            lcd.setCursor(0, 1);
            lcd.print("Sin ");
            break;
          case 1:
            lcd.setCursor(0, 1);
            lcd.print("Tri ");
            break;
          case 2:
            lcd.setCursor(0, 1);
            lcd.print("Saw ");
            break;
          case 3:
            lcd.setCursor(0, 1);
            lcd.print("Squ ");
            break;
          case 4:
            lcd.setCursor(0, 1);
            lcd.print("Us1 ");
            break;
          case 5:
            lcd.setCursor(0, 1);
            lcd.print("Us2 ");
            break;
          case 6:
            lcd.setCursor(0, 1);
            lcd.print("Us3 ");
            arrow(3, 1);
            break;
        }
        if (lfoLowRange)
        {
          showValue(4, 1, 255 - userLfoRate);
          lcd.setCursor(8, 1);
          lcd.print("Low ");
        }
        else // high range
        {
          showValue(4, 1, 256 - (userLfoRate >> 2));
          lcd.setCursor(8, 1);
          lcd.print("Hi  ");
        }
        if (retrigger)
        {
          lcd.setCursor(12, 1);
          lcd.print("On  ");
        }
        else // retrigger is false
        {
          lcd.setCursor(12, 1);
          lcd.print("Off ");
        }
        break;

      case 51: // OSCILLATORS - change user waveshape
        lcd.setCursor(0, 0);
        lcd.print("Us3: ");

        if (inFolder)
        {
          lcd.setCursor(4, 0);
          lcd.print(folderName);
          lcd.print("        ");
        }
        else
        {
          lcd.setCursor(4, 0);
          lcd.print("Root");
          lcd.print("        ");
        }
        getWaveform();
        break;

      case 60: // MODULATION
        static byte lastSource = 255;
        lcd.setCursor(0, 0);
        lcd.print("Src Dst Amt    ");
        switch (source)
        {
          case 0: // SOURCE LFO
            if (lastSource != source)
            {
              lcd.setCursor(0, 1);
              lcd.print("                ");
              lastSource = source;
            }
            lcd.setCursor(0, 1);
            lcd.print("LFO ");

            switch (destination)
            {
              case 0:
                lcd.setCursor(4, 1);
                lcd.print("Os1 ");
                showValue(8, 1, lfoOsc1DetuneFactor >> 2);
                break;
              case 1:
                lcd.setCursor(4, 1);
                lcd.print("Os2 ");
                showValue(8, 1, lfoOsc2DetuneFactor >> 2);
                break;
              case 2:
                lcd.setCursor(4, 1);
                lcd.print("Oc1 ");
                showValue(8, 1, osc1OctaveMod);
                break;
              case 3:
                lcd.setCursor(4, 1);
                lcd.print("Oc2 ");
                showValue(8, 1, osc2OctaveMod);
                break;
              case 4:
                lcd.setCursor(4, 1);
                lcd.print("Cut ");
                showValue(8, 1, lfoAmount >> 2);
                break;
              case 5:
                lcd.setCursor(4, 1);
                lcd.print("Amp ");
                showValue(8, 1, lfoAmpFactor >> 2);
                break;
              case 6:
                lcd.setCursor(4, 1);
                lcd.print("PWi ");
                showValue(8, 1, lfoPwFactor >> 2);
                break;
            }
            break;

          case 1: // SOURCE ENVELOPE
            {
              if (lastSource != source)
              {
                lcd.setCursor(0, 1);
                lcd.print("                ");
                lastSource = source;
              }
              lcd.setCursor(0, 1);
              lcd.print("Env ");
              switch (destination)
              {
                case 7:
                  lcd.setCursor(4, 1);
                  lcd.print("Os1 ");
                  showValue(8, 1, envOsc1PitchFactor >> 4);
                  break;
                case 8:
                  lcd.setCursor(4, 1);
                  lcd.print("Os2 ");
                  showValue(8, 1, envOsc2PitchFactor >> 4);
                  break;
                case 9:
                  lcd.setCursor(4, 1);
                  lcd.print("Cut ");
                  showValue(8, 1, envFilterCutoffFactor >> 4);
                  break;
                case 10:
                  lcd.setCursor(4, 1);
                  lcd.print("Rt ");
                  showValue(8, 1, envLfoRate >> 2);
                  break;
              }
            }
            break;
        }
        break;

      case 70: // SYNTH LOAD
        lcd.setCursor(0, 0);
        lcd.print("Syn: ");

        if (inFolder)
        {
          lcd.setCursor(4, 0);
          lcd.print(folderName);
          lcd.print("        ");
        }
        else
        {
          lcd.setCursor(4, 0);
          lcd.print("Root");
          lcd.print("        ");
          lcd.setCursor(0, 1);
          lcd.print("    Choose [P4] ");
        }
        getSynthPatch();
        break;

      case 80: // SYNTH SAVE
        lcd.setCursor(0, 0);
        lcd.print("Syn: ");

        if (inFolder)
        {
          lcd.setCursor(4, 0);
          lcd.print(folderName);
          lcd.print("        ");
        }
        else
        {
          lcd.setCursor(4, 0);
          lcd.print("Root");
          lcd.print("        ");
          lcd.setCursor(0, 1);
          lcd.print("    Choose [P4] ");
        }
        saveSynthPatch();
        break;
    }
  }
}
void arrow(byte x, byte y)
{
  arrowAnimation = true;
  arrowX = x;
  arrowY = y;
}

void arrowAnim()
{
  static byte lastArrowFrame = 255; // can't initialize to 0
  byte arrowMap[10] = {0, 0, 0, 0, 0, 1, 2, 3, 2, 1};
  if (arrowAnimation && arrowFrame != lastArrowFrame)
  {
    lastArrowFrame = arrowFrame;
    lcd.setCursor(arrowX, arrowY);
    switch (arrowMap[arrowFrame])
    {
      case 0:
        lcd.write(byte(0));
        break;
      case 1:
        lcd.write((byte) 1);
        break;
      case 2:
        lcd.write(2);
        break;
      case 3:
        lcd.write(3);
        break;
    }
  }
}

void showTB2(byte x) // horizontal posiztion
{
  lcd.setCursor(x, 0);
  lcd.write(2);
  lcd.write(4);
  lcd.write(6);
  lcd.setCursor(x, 1);
  lcd.write(3);
  lcd.write(5);
  lcd.write(7);
}
